<?php


class WC_DirectPayV3_Gateway extends WC_Payment_Gateway
{

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->id = 'WC_DirectPayV3_Gateway';
        $this->method_title = 'DirectPay V3';
        $this->method_description = 'WooCommerce Payment Plugin of DirectPay Payment Gateway V3.';
        $this->icon = 'https://cdn.directpay.lk/live/gateway/dp_woo_logo_wide.png';
        $this->has_fields = false;

        // gateways can support subscriptions, refunds, saved payment methods,
        $this->supports = array(
            'products'
        );

        // Method with all the options fields
        $this->init_form_fields();

        // Load the settings.
        $this->stage = [
            'local' => 'LOCAL',
            'dev' => 'DEV',
            'prod' => 'PROD'
        ];

        $this->init_settings();
        $this->enabled = $this->get_option('enabled');
        $this->testmode = 'yes' === $this->get_option('test_mode');
        $this->selected_stage = $this->stage['prod'];

        $test_title = '';
        $test_description = '';
        if ($this->testmode == 'yes') {
            $test_title = '';
            $test_description = '<br/><br/><b>(Developer Mode is Active. You will not be charged.)</b><br/>';
            $this->selected_stage = $this->stage['dev'];
        }

        $this->title = $this->get_option('title') . $test_title;
        $this->description = $this->get_option('description') . $test_description;
        $this->merchant_id = $this->get_option('merchant_id');
        $this->secret = $this->get_option('secret');
        //$this->redirect_page = $this->get_option('redirect_page');
        $this->logo_url = $this->get_option('logo_url');

        add_action('init', array(&$this, 'check_directPayV3_response'));
        add_action('woocommerce_api_' . strtolower(get_class($this)), array($this, 'check_directPayV3_response')); //update for woocommerce >2.0

        add_action('woocommerce_gateway_icon', array($this, 'modify_gateway_icon_css'), 10, 2);

        if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        } else {
            add_action('woocommerce_update_options_payment_gateways', array($this, 'process_admin_options'));
        }

        add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));
    }

    /**
     * Plugin options
     */
    function init_form_fields()
    {

        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable/Disable', 'woo_directPayV3'),
                'type' => 'checkbox',
                'label' => __('Enable DirectPay', 'woo_directPayV3'),
                'default' => 'yes',
                'description' => 'Show in the Payment List as a payment option'
            ),
            'title' => array(
                'title' => __('Title', 'woo_directPayV3'),
                'type' => 'text',
                'default' => __('DirectPay', 'woo_directPayV3'),
                'description' => __('This controls the title which the user sees during checkout.', 'woo_directpay'),
                'desc_tip' => true
            ),
            'description' => array(
                'title' => __('Description:', 'woo_directPayV3'),
                'type' => 'textarea',
                'default' => __('Pay by Visa, MasterCard, Amex, Frimi, Ipay and Lanka-QR via DirectPay.', 'woo_directPayV3'),
                'description' => __('This controls the description which the user sees during checkout.', 'woo_directPayV3'),
                'desc_tip' => true
            ),
            'merchant_id' => array(
                'title' => __('Merchant ID', 'woo_directPayV3'),
                'type' => 'text',
                'description' => __('Your DirectPay Merchant ID'),
                'desc_tip' => true
            ),
            'secret' => array(
                'title' => __('Secret Key', 'woo_directPayV3'),
                'type' => 'text',
                'description' => __('Your DirectPay Secret Key'),
                'desc_tip' => true
            ),
            'test_mode' => array(
                'title' => __('Sandbox Mode', 'woo_directPayV3'),
                'type' => 'checkbox',
                'label' => __('Enable Test Mode', 'woo_directPayV3'),
                'default' => 'yes',
                'description' => __('DirectPay test mode can be used to test payments before production', 'woo_directPayV3'),
                'desc_tip' => true
            ),
            'logo_url' => array(
                'title' => __('Logo Url', 'woo_directPayV3'),
                'type' => 'text',
                'description' => __('DirectPay checkout window logo'),
                'desc_tip' => true
            ),
        );

    }

    /**
     * Get Page list from WordPress
     **/
    function get_wordpress_page_list($title = false, $indent = true): array
    {
        $wp_pages = get_pages('sort_column=menu_order');
        $page_list = array();
        if ($title) $page_list[] = $title;
        foreach ($wp_pages as $page) {
            $prefix = '';
            // show indented child pages?
            if ($indent) {
                $has_parent = $page->post_parent;
                while ($has_parent) {
                    $prefix .= ' - ';
                    $next_page = get_post($has_parent);
                    $has_parent = $next_page->post_parent;
                }
            }
            // add to page list array array
            $page_list[$page->ID] = $prefix . $page->post_title;
        }
        return $page_list;
    }

    function modify_gateway_icon_css($icon_html, $payment_gateway_id)
    {
        if ($payment_gateway_id != $this->id) {
            return $icon_html;
        }

        $new_css = 'style="float: none;max-height: none;width: 100%;" src';
        $icon_html = preg_replace('/(src){1}/', $new_css, $icon_html);

        return $icon_html;
    }

    /**
     * Output settings in the correct format.
     */
    public function admin_options()
    {
        echo '<h3>' . __('DirectPay', 'woo_directPayV3') . '</h3>';
        echo '<p>' . __('WooCommerce Payment Plugin of DirectPay Payment Gateway Version 3.') . '</p>';
        echo '<table class="form-table">';
        $this->generate_settings_html();
        echo '</table>';
    }

    function receipt_page($order)
    {
        echo '<p><strong>' . __('Thank you for your order.', 'woo_directPayV3') . '</strong><br/><b>' . __('The payment page will open soon.', 'woo_directPayV3') . '</b></p>';
        echo $this->generate_directPayV3_form($order);
    }

    function generate_directPayV3_form($order_id)
    {
        global $woocommerce;
        $order = new WC_Order($order_id);
        //$orderDetails = $order->get_data();
        //$order->update_status('wc-pending-dp', __('Awaiting Directpay Payment', 'woocommerce'));

        $productName = "";
        $orders_arr = $order->get_items();
        if (count($orders_arr) > 1) {
            $productName = count($orders_arr) . " products";
        } else {
            $productName = "1 Product";
        }

        $redirect_url = $order->get_checkout_order_received_url();
        // Redirect URL : For WooCoomerce 2.0
        if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
            $notify_url = add_query_arg('wc-api', get_class($this), $redirect_url);
        }

        $orderId = $this->merchant_id . '-' . $order->get_order_number() . '-' . date("ymds");

        $payRequest = [
            "merchant_id" => $this->merchant_id,
            "amount" => (string)$order->get_total(),
            "type" => 'ONE_TIME',
            "order_id" => $orderId,
            "currency" => get_woocommerce_currency(),
            "return_url" => "",
            "response_url" => $notify_url,
            "first_name" => $order->get_billing_first_name(),
            "last_name" => $order->get_billing_last_name(),
            "phone" => $order->get_billing_phone(),
            "email" => $order->get_billing_email(),
            "page_type" => 'IN_APP',
            "logo" => $this->logo_url,
            "description" => $productName,
        ];

        $dataString = base64_encode(json_encode($payRequest));
        $signature = hash_hmac('sha256', $dataString, $this->secret);
        $stage = $this->selected_stage;
        ob_start();
        include plugin_dir_path(__FILE__) . '../templates/checkout.php';
        return ob_get_clean();
    }

    private function addOrderNotes($orderId, $note)
    {

        global $woocommerce;
        $order = new WC_Order($orderId);
        error_log('addOrderNotes' . $orderId . '-' . $note . "\n");
        if (!empty($note)) {
            $order->add_order_note('[DirectPayV3][Client-log][' . $orderId . ']:' . $note);
        }

    }

    public function createClientNote()
    {
        $order_id = $_REQUEST['order_id'];
        $note = $_REQUEST['note'];

        global $woocommerce;
        $order = new WC_Order($order_id);
        error_log('createClientNote: ' . $order_id . '-' . $note . "\n");
        if (!empty($note)) {
            $order->add_order_note('[DirectPayV3][Client-log][' . $order_id . ']:' . $note);
        }

        echo json_encode([
            'status' => 'success'
        ]);
        exit();
    }

    /**
     * You will need it if you want your custom credit card form
     */
    public function payment_fields()
    {

        if ($this->description) {
            echo wpautop(wptexturize($this->description));
        }

    }

    function check_directPayV3_response()
    {
        global $woocommerce;
        $fun = '[DirectPayV3][Server-Response]: ';
        error_log($fun . 'Start');
        $postBody_raw = file_get_contents('php://input');
        $postBody = json_decode(base64_decode($postBody_raw), true);
        $success = false;

        $headers = array();
        foreach ($_SERVER as $key => $value) {
            if (substr($key, 0, 5) <> 'HTTP_') {
                continue;
            }
            $header = str_replace(' ', '-', ucwords(str_replace('_', ' ', strtolower(substr($key, 5)))));
            $headers[$header] = $value;
        }

        error_log($fun . 'postBody_raw-' . $postBody_raw);
        error_log($fun . 'postBody-' . json_encode($postBody));
        error_log($fun . 'headers-' . json_encode($headers));

        $transactionType = $postBody["type"];
        $orderId = $postBody["order_id"];
        $transactionId = $postBody["transaction_id"];
        $transactionStatus = isset($postBody["transaction"]) ? $postBody["transaction"]["status"] : "-";
        $transactionDesc = isset($postBody["transaction"]) ? $postBody["transaction"]["description"] : "-";
        $paymentAmount = isset($postBody["transaction"]) ? $postBody["transaction"]["amount"] : "0.00";
        $paymentCurrency = isset($postBody["transaction"]) ? $postBody["transaction"]["currency"] : "LKR";

        $order_id = explode('-', $orderId)[1];
        $order = new WC_Order($order_id);
        $orderDetails = $order->get_data();

        if (isset($headers['Authorization'])) {
            $authHeaders = explode(' ', $headers['Authorization']);
            if (count($authHeaders) == 2) {
                $hash = hash_hmac('sha256', $postBody_raw, $this->secret);
                if (strcmp($authHeaders[1], $hash) == 0) {
                    $success = true;
                    $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Signature Verified');
                } else {
                    $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Signature Verification Failed.');
                }
            } else {
                $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Invalid Signature. Headers:' . json_encode($headers) . '| Raw Headers: ' . json_encode($_SERVER));
            }
        } else {
            $success = true;
        }

        if ($success) {

            $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']:' . json_encode($postBody));

            if ($transactionStatus == 'SUCCESS') {
                $order->payment_complete();
                //$order->update_status('wc-pending-approval');
                $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Payment successful. 
                <br/> Transaction ID:' . $transactionId);
            } else {
                $order->update_status('failed');
                $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Payment failed. 
                <br/> Transaction ID: ' . $transactionId . '<br/> Description: ' . $transactionDesc);

            }

        } else {
            $order->update_status('on-hold');
            $order->add_order_note('[DirectPayV3][Server-log][' . $order_id . ']: Signature Verification Failed status changed to <b>on-hold</b>');
        }

        $woocommerce->cart->empty_cart();
        error_log('[DirectPayV3][Server-Response]: End' . "\n");
    }


    /*
     * We're processing the payments here
     */
    public function process_payment($order_id)
    {

        global $woocommerce;
        $order = new WC_Order($order_id);

        if (version_compare(WOOCOMMERCE_VERSION, '2.1.0', '>=')) { // For WC 2.1.0
            $checkout_payment_url = $order->get_checkout_payment_url(true);
        } else {
            $checkout_payment_url = get_permalink(get_option('woocommerce_pay_page_id'));
        }

        return array(
            'result' => 'success',
            'redirect' => add_query_arg(
                'order',
                $order->get_id(),
                add_query_arg(
                    'key',
                    $order->order_key,
                    $checkout_payment_url
                )
            )
        );
    }

    /*
     * In case you need a webhook
     */
    public function webhook()
    {


    }
}